/*************************************************************************************
 * Copyright (C), 2004
 * SAS Institute Inc., Cary, N.C. 27513, U.S.A.  All rights reserved.
 * 
 *    NAME: orva_ffitkri_dist.sas                                             
 * PURPOSE: sub-macro of %orva_freq_fit_kri() called once for each distribution.            
 * SUPPORT: lajack
 *
 * PARAMETERS:
 *    FDIST       = Current distribution.
 *    NDIST       = Current distribution number.
 *
 * NOTES: Please see the accompanying doc for %orva_freq_fit_kri().
 *        The output of this macro is the datasets:
 *                   countout_est_&ndist
 *                   fbluidesta_&ndist
 *                   CovMat_&ndist
 **************************************************************************************/
%macro orva_ffitkri_dist(fdist=,ndist=);

    ods listing close;

	%let reason=Successful;
	%let abort=0;
    data countout_est_&ndist; set _NULL_; run;
    data fbluidesta_&ndist; set _NULL_; run;
    data CovMat_&ndist; set _NULL_; run;
     
	%let try=1;
	%let nl_opt_methods = NEWRAP TRUREG ; /* QUANEW has a bug in it, in Countreg 913sp3.  
	To wit, it doesn't calc cov mat correctly, when only one iteration is done, or when a restart happens. */
	%let curr_method = %scan(&nl_opt_methods, &try, ' ');
	
	%let initialalpha = ; 
    %if %upcase(&fdist) EQ NEGBIN %then %do; 
        %let freqModelType = NEGBIN1; 
        %let extraColumns = 2;
        %let possiblyAlpha = Beta;
		proc summary data=_kriblLossespobs_ ;
			var newfrequency;
			by bluid;
			output out=summaryoutpobs mean=the_mean var=the_var;
			weight combinedexposure;
		run;
		data summaryoutpobs(drop=initalpha /*sum_mean sum_var*/);
			set summaryoutpobs end=last;
			sum_mean + the_mean;
			sum_var + the_var;
			if last then do;
				if (sum_var < sum_mean) then initalpha = .000001;
				else initalpha = sum_var/sum_mean - 1;
				if (initalpha GE 3) then initalpha = 3;
				call symput("initialalpha", initalpha);
			end;
		run;
    %end;
    %else %do;
        %let freqModelType = POISSON;
        %let extraColumns = 1;
        %let possiblyAlpha = ;
    %end;

     /* Get an initial guess from genmod */
    data _kriblLossespobs_; 
		set _kriblLossespobs_;
        keep newfrequency &SelectedKRIs &possiblyDummyVars offset bluid combinedexposure pScaledExcess;
    run;

%if (&SelectedKRIs NE ) OR (&possiblyDummyVars NE ) %then %do;
    data _NULL_;
        length path $ 256;
        path = pathname("work");
        call symput('path',trim(path));
    run;
	%if &genmod_results_exist EQ 1 %then %goto MAKE_INIT_STATEMENT;
    proc contents data=_kriblLossespobs_(drop=newfrequency offset bluid combinedexposure pScaledExcess) noprint out=_foo;
    run;
    %if &SYSERR NE 0 & &SYSERR NE 4 %then %do;
        %put ERROR: Unable to obtain path for temporary workspace.;
        %let abort=1;
        %let reason=Unable to obtain path for temporary workspace;
        %goto DONE;
    %end;    
    filename tmp "&path\genmod_tmp";
	data _null_;
        set _foo end=last;
        file tmp;
        if _N_ = 1 then do;
		 	put 'proc genmod data=_kriblLossespobs_;';
			put 'model newfrequency= ';
		end;
		put name;
		if last then do;
			put ' / offset=offset dist=poisson link=log;';
			put 'run;';
		end;
    run;
    %if &SYSERR NE 0 & &SYSERR NE 4 %then %do;
        %put ERROR: Unable to create or to write to code file &path/genmod_tmp;
        %let abort=1;
        %let reason=Unable to create or to write to code file &path/genmod_tmp.;
        %goto DONE;
    %end;
	ods output ParameterEstimates=countouta_ ;
    /* run the program we just made */
    %inc "&path\genmod_tmp";
    %if &SYSERR NE 0 & &SYSERR NE 4 %then %do;
        %put ERROR: Unable to run code in &path/genmod_tmp.;
        %let abort=1;
        %let reason=Unable to run code in &path/genmod_tmp;
        %goto DONE;
    %end;
	%let genmod_results_exist = 1;

    %MAKE_INIT_STATEMENT:
	filename tmp "&path\initial_guess_sas_tmp";
	data _NULL_; 
		format estimate 32.15;
		set countouta_;
		file tmp;
		if (_n_ = 1) then do; 
			put "init ";
			%if &freqModelType EQ NEGBIN1 %then %do;
				put " _alpha &initialalpha" ; 
			%end;
		end;
		if parameter ^= "Scale" and parameter ^= "Dispersion" then do;
			put parameter " " estimate;
		end;		
	run;
%end;

 
	%TRYAGAIN:

        /* proc countreg */
        ods output FitSummary =  _fitsummary_ ; 
        proc countreg data=_kriblLossespobs_ outest=countouta_&ndist covout type=&freqModelType method=&curr_method;
/*		proc countreg data=_kriblLossespobs_ outest=countouta_&ndist covout type=&freqModelType method=&curr_method  inhess=1;*/
        %if (&SelectedKRIs NE ) OR (&possiblyDummyVars NE ) %then %do;
 			%inc "&path\initial_guess_sas_tmp"; ;
		%end;
		%else %if &freqModelType EQ NEGBIN1 %then %do;
			init _alpha &initialalpha;
		%end;

            /* CANNOT USE ARRAY statement WITH COUNTREG!!! */
            model newfrequency= &SelectedKRIs &possiblyDummyVars  / offset=offset;
        run;
		data _null_;
			set countouta_&ndist;
			if _N_ = 1 then	call symput("nl_opt_status", substr(_STATUS_,1,2));
		run;
		%if &nl_opt_status NE 0  %then %do;
			%let try=%eval(&try+1);
			%if &try LT 3 %then %do;  /* Change 3 to 4 when list of methods includes QUANEW, NEWRAP, TRUREG */
				%let prev_method = &curr_method;
				%let curr_method = %scan(&nl_opt_methods, &try, ' ');
				%put ERROR: &freqModelType: &prev_method method failed. Trying again with &curr_method method.;
				%goto TRYAGAIN;
			%end;
			%else %do;
				%put ERROR: &freqModelType: &curr_method method failed.  Frequency fitting failed.;
				%let reason = &freqModelType: Newton-Raphson and trust region methods failed.;
				%let abort = 3;
				 %goto DONE;
			%end;
		%end;

        proc transpose data=_fitsummary_ out=_fitsummary_;
            id label1; 
            var nValue1;
        run;

        data  countout_est_&ndist(drop=&possiblyDummyVars _NAME_ _TYPE_) 
                 fcov_&ndist(drop=_NAME_ _TYPE_ _STATUS_ CCUID) 
				%if %eval(&numSelectedBLs) GT 0 %then %do;
                    fbluidesta_&ndist(drop= &SelectedKRIs Intercept _NAME_ _TYPE_ _STATUS_ Distribution CCUID)
				%end;
				 ;
            CCUID = &ccUid;
            length Distribution $12;
            Distribution = upcase("&fdist");
            set countouta_&ndist;
            if _N_ EQ 1 then do;
                output countout_est_&ndist;
				%if %eval(&numSelectedBLs) gt 0 %then %do; 
					output fbluidesta_&ndist;
				%end;
            end;
            else do;
                output fcov_&ndist;
            end;
        run;

        /* adjust the covariance matrix to get rid of the dummys */
        proc iml;
            use fcov_&ndist;
            read all var _num_ into C;
            close fcov_&ndist;
            X = ginv(C);

            /* Create a new matrix that includes only these variables:  
            "Intercept" and "KRI_:" and possibly Alpha (if &fdist=Negbin),
            and the corresponding rows.  */

            G = X[1:&numSelectedKRIs+&extraColumns,1:&numSelectedKRIs+&extraColumns]; 
                /* +1 is for Intercept,  +2 is for Intercept and alpha */

            new_cov = ginv(G);

            names = {Intercept &selectedKRIs &possiblyAlpha}; /* create a matrix of column names */
            _NAME_ = {Intercept &selectedKRIs &possiblyAlpha}; /* create a matrix of row names */
            /* Create the data set CovMat_&ndist from the matrix new_cov : */
            create CovMat_&ndist from new_cov [colname=names rowname=_NAME_];
            /* Must include the same "rowname=" matrix for every subsequent append statement : */
            append from new_cov [rowname=_NAME_]; 
        quit;

        data CovMat_&ndist;
            length _NAME_ $32;
            length Distribution $12;
            Distribution = upcase("&fdist");
            set CovMat_&ndist;
        run;

		%if %eval(&numSelectedBLs) GT 0 %then %do;
			proc transpose data=fbluidesta_&ndist out=fbluidest_&ndist(rename=(COL1=Estimate));
		        var dummy_bluid_:;
	    	run;
	    	data fbluidest_&ndist(drop=_NAME_) ;
	    	    length bluid 8;
	       	 	set fbluidest_&ndist &notSelected; /* ???? Check whether &notSelected exists or has obs */
		    	/* Only output observations from &notSelected that begin with "dummy_bluid_" .
				That is, we don't want any non-selected KRIs appearing in this output dataset.  */
	        	if substr(_NAME_,1,12) eq 'dummy_bluid_' then do;
	            	bluid = scan(_NAME_, -1, "_");
	            	output;
	        	end;  
	    	run;
			data fbluidest_&ndist;
				CCUID = &ccUid;
        		length Distribution $12;
       	 		length bluid 8;
       	 		Distribution = upcase("&fdist");
				if _n_ EQ 1 then do;
	       	 		bluid = &extraBL; 
					Estimate = .; 
					output;
				end;
				set fbluidest_&ndist; 
				output;					
			run;
		%end;
		%else %if %eval(&numBLUIDsThisCell) GT 1 %then %do; /* NotSelected is non-empty */
			data fbluidest_&ndist;
				length bluid 8;
	       	 	set &notSelected;
		    	/* Only output observations from &notSelected that begin with "dummy_bluid_" .
				That is, we don't want any non-selected KRIs appearing in this output dataset.  */
	        	if substr(_NAME_,1,12) eq 'dummy_bluid_' then do;
	            	bluid = scan(_NAME_, -1, "_");
	            	output;
	        	end;  
	    	run;
			data fbluidest_&ndist;
				CCUID = &ccUid;
        		length Distribution $12;
       	 		length bluid 8;
       	 		Distribution = upcase("&fdist");
				if _n_ EQ 1 then do;
	       	 		bluid = &extraBL;
					Estimate = .; 
					output;
				end;
				set fbluidest_&ndist; 
				output;					
			run;

		%end;
		%else %do; /* There is only one BL in this cell, so no Dummy Vars were ever considered for selection */
			data fbluidest_&ndist;
				CCUID = &ccUid;
	        	length Distribution $12;
	       	 	length bluid 8;
	       	 	Distribution = upcase("&fdist");
	       	 	bluid = &extraBL;  
				Estimate = .;
	    	run;
		%end;
		
%DONE:
    %if &abort %then %do;
        data countout_est_&ndist;
            format status 8. reason $100.;
			length Distribution $12;
            status=1;
            reason="&reason";
            Distribution = upcase("&fdist");
        run;
		data fbluidest_&ndist;
	    run;
		data CovMat_&ndist;
		run;

    %end;   
    %else %do; 
        data countout_est_&ndist;
            format status 8. reason $100.;
            status=0;
            reason="&reason";
            set _fitsummary_(rename=(Log_Likelihood=LogL));
            set countout_est_&ndist;
            drop Dependent_Variable _NAME_;
        run;
    %end;
    
/*	proc datasets library=work memtype=data nolist nodetails;*/
/*    	delete  */
/*    	  countouta_&ndist*/
/*          fcov_&ndist*/
/*        ;*/
/*	run;*/
        
%mend orva_ffitkri_dist;
