/*****************************************************************************************
 *  Copyright (C), 2005
 *  SAS Institute Inc., Cary, N.C. 27513, U.S.A.  All rights reserved.
 * 
 *  NAME:    orva_kri_subset.sas                                             
 *  PURPOSE: Prepares the kri dataset for a project by renaming the firm-base kris 
 *           kri_F.
 *  SUPPORT: stchri
 *
 *  PARAMETERS:
 *    kriData     = name of the denormalized KRI dataset (Meta) to subset.
 *    bl          = true - handling Business Line KRIs
 *                  false - handling Firm KRIs.
 *    kriOut      = name of the output KRI dataset.
 *    ostat       = name of the out status dataset.  Will contain the following:
 *                    status = 0 on success, 1 otherwise.
 *                    reason = printable message.
 *    mapDataPrefix = NOT USED!!!
 *
 * NOTES: see sample code at end of file.
 *      
 *****************************************************************************************/
%macro orva_kri_subset( kriData       =,
                        bl            =true,
                        kriOut        =,
                        ostat         =,
                        mapDataPrefix =);
                         

                      
   /*** create the status dataset up front ***/
   %let status=0;
   %let reason=Successful;
   %let abort=0;
   data &ostat;
        format status 8. reason $100.;
        status=1;
        reason="Unknown";
   run;

    /******************** FIRM data *************************/
	%if &bl ne true %then %do;
	   /*** create macro variable with all kri names in it ***/
	   %let kriNames= ;
	   proc contents data=&kriData out=_c noprint; run;
	   data _NULL_; set _c;
	    retain krivars;   
	    length krivars $ 1024;
	    if upcase(substr(NAME,1,4)) eq 'KRI_' then do;
	       found = 1;
	       krivars = trim(krivars) || ' ' || trim(name);
	       call symput( 'kriNames', krivars );      
	     end;
	   run;
	      
	   /*** rename kris for firm with _F ***/
	   data &kriOut; 
	     set &kriData 
              (rename=(
	 	         %let nkris=1;
		         %let kri=%scan(&kriNames, &nkris, ' ');
		         %do %while (&kri ne ); 
		            &kri=&kri._F
		            %let nkris=%eval(&nkris+1);
		            %let kri=%scan(&kriNames, &nkris, ' ');
		         %end;   
		      ))
	          
	     ;
	   run;
	   
	   /*** cleanup temporary datasets ***/
	   proc datasets library=work memtype=data nolist nodetails;
	      delete _c;
	   run; quit;
	   
	%end;
	
    /******************** BL data *************************/
	%else %do; 
	      data &kriOut; 
	         set &kriData;
	      run;
	%end;
   
 %DONE:
    %if &abort %then %do;
	    %put ERROR: orva_kri_subset() macro ended abnormally.;
	    %let status = 1;
	%end;
	data &ostat;
	    format status 8. reason $100.;
	    status=&status;
	    reason="&reason";
	run;

%mend orva_kri_subset;


/***************************************************
%orva_kri_subset( kriData= kribl2,
                  bl= true,
                  kriOut= kriblprj,
                  ostat= ostat,
                  mapDataPrefix=VARPRJ.CCMAPI_);
%orva_kri_subset( kriData= krifirm,
                  bl= false,
                  kriOut= krifirmprj,
                  ostat= ostat,
                  mapDataPrefix=VARPRJ.CCMAPI_);
****************************************************/
