/*************************************************************************************
 * Copyright (C), 2004
 * SAS Institute Inc., Cary, N.C. 27513, U.S.A.  All rights reserved.
 * 
 *    NAME: orva_sfit_dist.sas                                             
 * PURPOSE: sub-macro of %orva_sfitall() called once for each distribution.            
 * SUPPORT: jariou
 *
 * PARAMETERS:
 *    DIST        = Current distribution.
 *    NIDST       = Current distribution number.
 *
 * NOTES: Please see the accompanying doc for %orva_sfitall().
 *        The only output of this macro is the dataset _parmnlp_&nidst.
 *
 **************************************************************************************/
%macro orva_sfit_dist(dist=,ndist=);
        
        %let NEEDED_INPUT=.;
        %let PARAM_COUNT=.;
        %let AVG_P_OBS = -1; /* initializing */

        /* load macros for this dist */
        %orvd_&dist;

        /*  Loop to check that all the extra UserInput is available */
        %let nNeededInput=1;
        %let NeededInput=%scan(&NEEDED_INPUT, &nNeededInput, ' ');
        data _null_;
            AllThere= 1;
            %do %while (&NeededInput ne );  
                AllThere = AllThere * symexist("&NeededInput"); 
                %let nNeededInput=%eval(&nNeededInput+1);
                %let NeededInput=%scan(&NEEDED_INPUT, &nNeededInput, ' ');
            %end;
            call symput("AllUserInput",AllThere);           
        run;
        /*  End of Loop to check that all the extra UserInput is available */


        %if %eval(&AllUserInput =0) %then %do;
            %put ERROR: Missing user input for model &dist, this model will not be fitted.;
            %let reason=Missing user input for model &dist;
            %let status=1;
            %goto LOCAL_EXIT;
        %end;

        %if %sysevalf(&PARAM_COUNT > &obsCount,boolean) %then %do;
            %put ERROR: Less data points than parameters.;
            %let reason=Less data points than parameters;
            %let status=1;
            %goto LOCAL_EXIT;
        %end;

        %let status=0;
        %let reason=Successful;

        
        %let param_1_init=.;
        %let param_2_init=.;
        %let param_3_init=.;
        %let param_4_init=.;
        %let nparms= 0;
        %InitFit&dist;
        %Fit&dist;
        
        %if &status=0 %then %do;
           %let nobs=0;
           %let dsid=%sysfunc(open(_cs_,I));
           %if &dsid ^= 0 %then %do;
              %let nobs=%sysfunc(attrn(&dsid,nlobs));
              %let rc=%sysfunc(close(&dsid));
           %end;
           %if &nobs eq 0 %then %do;
              %let status = 1;
              %let reason = Unknown convergence failure.;
           %end;
           %else %do;
            data _null_;
                set _cs_;
                    if (substr(reason,1,7) eq "WARNING") then status = 0;
                call symput("status",status);
                call symput("reason",reason);
            run;
        %end;
        %end;

        %if &status=0 %then %do;
            /*- have to make the loglikelihood comparable for LNIG and LNG -*/
            %let sumlog= 0;
            %MakeLLGood&dist;
            data _null_;
                set _logl_;
                if _N_=3 then do;
                call symput("logl",nValue1);
                aic = -2*(nValue1 - &sumlog - &nparms);
                sbc = -2*(nValue1 - &sumlog - &nparms * log(&NExcess)/2);
                call symput("aic",aic);
                call symput("sbc",sbc);
                end;
            run;

            data _null_;
                set _covmat_ nobs=nobs;
                call symput("nobs",left(nobs));
                call symput("ncov",left(trim(nobs*(nobs+1)/2)));
                call symput(compress("row"||_N_),rowname);
            run;

            data _tmp_;
                set _covmat_;

                array covs[&ncov] cov_1-cov_&ncov;
                array params[&nobs] Param_1-Param_&nobs;
                retain cov_:;

                do i=1 to _N_;
                    covs(_N_*(_N_-1)/2+i)=params(i);
                end;
            run;

            data _tmp_;
                set _tmp_ (drop =rowname i param_:);
                if _N_ ne &nobs then delete;
            run;    

            data _parmnlp_;
                format distribution $12. status 8. reason $100.;    
                
                set _parmnlp_ (where =(index(_type_,"PARMS")>0));
                set _tmp_;
                distribution="&dist";
                status=&status;
                reason="&reason";
                logl=&logl-&sumlog;
                aic=&aic;
                sbc=&sbc;
            run;

            %ORVA_GFit(data=_workData_,var=AMOUNT,thresholdVar=THRESHOLD,ccUid=&ccUid,dist=&dist,
                       parmdata=_parmnlp_,gofThreshold=&gofThreshold);  

            %let KRI_LOG_AVG = 0;

            data _parmnlp_(drop= m1 m2 m3 m4);
               set _parmnlp_;
               %RealMoments&dist;
            run;
              
            %NoMissingMOM&dist.(parmdata=_parmnlp_);

            /* Set the macro variables for the parameters of the fitted &dist: param_1, param_2, etc. */
            data _parmtemp_;
                set _parmnlp_(keep=_numeric_);
            run;

            proc transpose data=_parmtemp_ out=_parmtemp_;
            run;

            data _null_;
                set _parmtemp_;
                call symput(_name_, col1);
            run;

				/* calculate the avg_p_obs */
				data _workdata_ (drop=p_obs_sum p_obs_count);
					set _workdata_ end=last;
					retain p_obs_sum 0 p_obs_count 0;

					if THRESHOLD=0 then do;
						p_obs=1;
					end;
					else do;
						%Sdf&dist(THRESHOLD, p_obs); /* ????should this be kri_adj_threshold or threshold??? */
					end;
					/* Note: in orva_sfit_kri, kri_adj_threshold is set to threshold / exp(kri_log) */
					/* Note: in orva_freqfit_kri, %Sdf&sdist(THRESHOLD/exp(&KRI_LOG), pScaledExcess);  */
					if ^missing(p_obs) then do;
						p_obs_sum = p_obs_sum + p_obs;
						p_obs_count = p_obs_count + 1;
					end;

					if last then do;
						call symput("avg_p_obs", p_obs_sum / p_obs_count);
					end;
				run;
        %end;
        %else %do;
%LOCAL_EXIT:
            data _parmnlp_;
                format distribution $12. status 8. reason $100.; 
                distribution="&dist";
                status=&status;
                reason="&reason";
            run;    
        %end;

        data _parmnlp_&ndist;
            set _parmnlp_;
            obsCount=&obsCount;
            paramcount=&PARAM_COUNT;
            avg_p_obs = &AVG_P_OBS;
        run;


%mend orva_sfit_dist;

