/*****************************************************************************************
 * Copyright (C), 2004
 * SAS Institute Inc., Cary, N.C. 27513, U.S.A.  All rights reserved.
 * 
 * NAME:    ORVA_Var_Compress.sas                                             
 * PURPOSE: Compress the draws for the var to conserve space          
 * SUPPORT: sasdje
 *
 * PARAMETERS
 *
 * simData = the simulation dataset. Is replaced after this call
 *
 * NREPS    =  the number of simulation replications, must be a positive integer.   
 *
 * OUTSIM   =  the name of the replication output data set 
 *              Has columns CCUID Loss Value Density
 *             Value and density can be used to plot series.
 *
 *****************************************************************************************/

%macro ORVA_Var_Compress(  simData=,
			   nreps=,
			   outstat=
				);
	

    %let Num_obs_compressed= 500;
	%let status=0;
	%let reason=Successful;
	%let abort=0;

	/*- create the status data set upfront -*/
	data &outstat;
		format status 8. reason $100.;
		status=1;
		reason="Unknown";
	run;


	proc sort data=&simData out=_outsim_;
		by CCUID loss;
	run;

        /* Compress the OUTSIM Data set --------------------*/
        /* If, for some reason you need all this data, use it */
        /* before this step ---------------------------------*/
        /* The compressed form saves the last &Num_obs_compressed */
        /* values (the tail ) and uses KDE to compress the entire */
        /* distribution to &Num_obs_compressed points             */
       
    %if &num_obs_compressed > &nreps %then %let num_obs_compressed=&nreps;
    proc kde data=_outsim_ ;
       univar loss / out=_temp_(keep=CCUID Value density) 
                     ngrid=&Num_obs_compressed;
       by CCUID;
    run;
    data _work_; set _outsim_;
       by CCUID;
       retain cnt;
       if first.CCUID = 1 then cnt = 1;
       keep CCUID loss;
       if cnt > &nreps-&Num_obs_compressed then output;
       cnt + 1;
    run;
    data &SimData; set _temp_; set _work_;
       label loss="Last &Num_obs_compressed tail losses";
    run;
         

%DONE:

	%if &abort %then %do;
		%put ERROR: %orva_var_Compress macro ended abnormally.;
	%end;	
	data &outstat;
		format status 8. reason $100.;
		status=&status;
		reason="&reason";
	run;

	/*- delete unwanted data sets -*/
	proc datasets library=work memtype=data nolist nodetails;
		delete _outsim_ _temp_ _work_;
	quit;

%mend;


