/*****************************************************************************************
 * $Id: orvl_migrate_monitor_ds.sas,v 1.1.2.3.4.1 2006/10/11 15:48:07 stchri Exp $
 * Copyright (C), 2005
 * SAS Institute Inc., Cary, N.C. 27513, U.S.A.  All rights reserved.
 * 
 *   Name: ORVL_MIGRATE_MONITOR_DS.sas                                             
 *   Title: Migrate Monitor datasets to VaR 3.2 datasets
 * 
 *   Parameters
 * 
 *   Internal_dir  = Directory Containing the Monitor datasets
 *   Output_Lib   = A valid libname to where the resulting dataset are put
 *
 ****************************************************************************************/

%macro ORVL_MIGRATE_MONITOR_DS(internal_dir, Output_Lib, outStatus=ostatus);

   %local syntaxopt; %let syntaxopt = %sysfunc(getoption(SYNTAXCHECK));
   options SYNTAXCHECK;

   %orvl_cmn_global_variables();

   /*- create the status datasets upfront -*/
   data WORK.LOAD_STATUS_DETAILS;
      format WHO $32. STATUS 8. REASON $800.;
      WHO="MAIN";
      STATUS=1;
      REASON="Unknown";
   run;

   data &outStatus;
      format status 8. reason $800.;
      status=1;
      reason="Unknown";
   run;

   %let TYP_NM = Monitor Transformation;
   %let skipToDone = 0;
   
   %orvl_ds_exists(&internal_dir..RM_ASSESSMENT_PERIOD_DIM, "&TYP_NM", true);
   %if &ds_nobs eq 0 %then %let skipToDone = 1;
   %orvl_ds_exists(&internal_dir..RM_BASEL_BL_DIM, "&TYP_NM", true);
   %if &ds_nobs eq 0 %then %let skipToDone = 1;
   %orvl_ds_exists(&internal_dir..RM_BL_DIM, "&TYP_NM", true);
   %if &ds_nobs eq 0 %then %let skipToDone = 1;
   %orvl_ds_exists(&internal_dir..RM_BASEL_RISK_CAT_DIM, "&TYP_NM", true);
   %if &ds_nobs eq 0 %then %let skipToDone = 1;
   %orvl_ds_exists(&internal_dir..RM_RISK_CAT_DIM, "&TYP_NM", true);
   %if &ds_nobs eq 0 %then %let skipToDone = 1;
   %orvl_ds_exists(&internal_dir..RM_PROCESS_DIM, "&TYP_NM", true);
   %if &ds_nobs eq 0 %then %let skipToDone = 1;
   %orvl_ds_exists(&internal_dir..RM_CAUSE_DIM, "&TYP_NM", true);
   %if &ds_nobs eq 0 %then %let skipToDone = 1;
   %orvl_ds_exists(&internal_dir..RM_GEOGRAPHY_DIM, "&TYP_NM", true);
   %if &ds_nobs eq 0 %then %let skipToDone = 1;
   /* KRI's not finished 
   %orvl_ds_exists(&internal_dir..RM_KRI_DIM, "&TYP_NM", true);
   %if &ds_nobs eq 0 %then %let skipToDone = 1;
   */
   %orvl_ds_exists(&internal_dir..RM_INSURANCE_POLICY_DIM, "&TYP_NM", true);
   %if &ds_nobs eq 0 %then %let skipToDone = 1;
   %orvl_ds_exists(&internal_dir..RM_OP_RISK_EVENT, "&TYP_NM", true);
   %if &ds_nobs eq 0 %then %let skipToDone = 1;
   %orvl_ds_exists(&internal_dir..RM_FINANCIAL_IMPACT, "&TYP_NM", true);
   %if &ds_nobs eq 0 %then %let skipToDone = 1;

   %if &skipToDone eq 1 %then %do;
      data _null_;
         call symput("LOAD_STATUS", 2);
         call symput("LOAD_REASON", "Failed to transform data, required dataset(s) missing.");
      run;
      %goto DONE;
   %end;

   %let VAR_IMPACT_VARS_COUNT = 31;
 /* !!!!Keep in sync with rename list usage below!!!! */
   %let VAR_IMPACT_VARS1 = "BL_RK" "CREDIT_LOSS_FLG" "DISCOVERY_DT" "EVENT_DISCOVERY_DT" "EVENT_END_DT" "EVENT_START_DT";
   %let VAR_IMPACT_VARS2 = "EVENT_THRESHOLD" "FINANCIAL_IMPACT_ID" "FINANCIAL_IMPACT_RK" "GEOGRAPHY_RK" "IMPACT_DESC";
 /* !!!!Keep in sync with rename list usage below!!!! */
   %let VAR_IMPACT_VARS3 = "IMPACT_END_DT" "IMPACT_EXPECTED_AMT" "IMPACT_FINANCIAL_STATUS_CD" "IMPACT_INSURANCE_AMT" "IMPACT_NON_INSURANCE_AMT";
   %let VAR_IMPACT_VARS4 = "IMPACT_LOSS_AMT" "IMPACT_PROVISIONED_AMT" "IMPACT_RECOVERY_AMT" "IMPACT_START_DT" "IMPACT_TYPE_CD";
 /* !!!!Keep in sync with rename list usage below!!!! */
   %let VAR_IMPACT_VARS5 = "LAST_UPDATE_USER_RK" "LEGAL_ORG_RK" "MANAGEMENT_ORG_RK" "OP_RISK_EVENT_RK" "ORIGINATOR_USER_RK";
   %let VAR_IMPACT_VARS6 = "PROCESS_RK" "RISK_CAT_RK" "SOURCE_SYSTEM_CD" "THRESHOLD" "VALIDATION_STATE";
 /* !!!!Keep in sync with rename list usage below!!!! */
   %let VAR_IMPACT_VARS = &VAR_IMPACT_VARS1 &VAR_IMPACT_VARS2 &VAR_IMPACT_VARS3 &VAR_IMPACT_VARS4 &VAR_IMPACT_VARS5 &VAR_IMPACT_VARS6;
 /* !!!!Keep in sync with rename list usage below!!!! */

   proc datasets NOLIST lib=&internal_dir;
      contents data=RM_FINANCIAL_IMPACT out=WORK.lossCols;
   quit;

   data _null_;
      length msg $800. NAME $32. MEMNAME $32.;

      declare hash inHash(dataset: "WORK.lossCols", hashexp: 10);
      rc = inHash.defineKey("NAME");
      rc = inHash.defineData("MEMNAME");
      rc = inHash.defineDone();
      call missing(NAME, MEMNAME);

      array var_names[&VAR_IMPACT_VARS_COUNT] $32 (&VAR_IMPACT_VARS);

      msg = "The following required columns are missing in RM_FINANCIAL_IMPACT:";
      missing_var = 0;

      do i = 1 to &VAR_IMPACT_VARS_COUNT;
         NAME = var_names[i];
         rc = inHash.find();
         if (rc ^= 0) then do; /* Referenced item not found.  Don't output the data. */
            msg = trim(left(msg)) || " " || trim(left(var_names[i]));
            missing_var = 1;
         end;
      end;

      if (missing_var = 1) then do;
         call symput("LOAD_STATUS", 4);
         call symput("LOAD_REASON", msg);
         msg = "STATUS " || trim(left(msg));
         put msg;
      end;
   run;
   
   %if %TRIM(%LEFT(&LOAD_STATUS)) ne 1 %then %do;
      %goto DONE;
   %end;
   
   %put STATUS PROCESS START;
   %put STATUS Starting conversion of Monitor Reporting Mart to input datasets;
   
   %orvl_cmn_status_init();

   %put STATUS Determining assessment period table for global range;
   proc summary data=&internal_dir..RM_ASSESSMENT_PERIOD_DIM;
      output min= Start_Date_Min End_Date_Min
             max= Start_Date_Max End_Date_Max
             out= work.global_range_summary(keep=Start_Date_Min End_Date_Min Start_Date_Max End_Date_Max);
      var START_DT END_DT;
   run;

   %put STATUS;
   %put STATUS Generating global date range dataset;
   data &Output_lib..global_range (keep= Range_ID Start_Date End_Date);
      set work.global_range_summary (rename=(Start_Date_Min=Start_Date
                                             End_Date_Max=End_Date));
      Range_ID = 1; 
   run;
   
   %put STATUS;
   %put STATUS Handling basel business line dataset for standard business lines;
   %orvl_migrate_hier_item(&internal_dir..RM_BASEL_BL_DIM, &Output_lib..std_business_line_structure, 
                           std_BL_ID, std_BL_Name, 
                           BASEL_BL_RK, BASEL_BL_ID, BASEL_BL_NM, 
                           BASEL_BL_RK_, SOURCE_SYSTEM_CD);

   %put STATUS;
   %put STATUS Handling internal business line dataset;
   %orvl_migrate_hier_item(&internal_dir..RM_BL_DIM, &Output_lib..internal_business_line_structure, 
                           Internal_BL_ID, Internal_BL_Name, 
                           BL_RK, BL_ID, BL_NM, 
                           BL_RK_, SOURCE_SYSTEM_CD);

   %put STATUS;
   %put STATUS Handling basel risk category dataset for standard risk categories;
   %orvl_migrate_hier_item(&internal_dir..RM_BASEL_RISK_CAT_DIM, &Output_lib..std_risk_category_structure, 
                           std_RC_ID, std_RC_Name, 
                           BASEL_RISK_CAT_RK, BASEL_RISK_CAT_ID, BASEL_RISK_CAT_NM, 
                           BASEL_RISK_CAT_RK_, SOURCE_SYSTEM_CD);

   %put STATUS;
   %put STATUS Handling internal risk category dataset;
   %orvl_migrate_hier_item(&internal_dir..RM_RISK_CAT_DIM, &Output_lib..internal_risk_category_structure, 
                           Internal_RC_ID, Internal_RC_Name, 
                           RISK_CAT_RK, RISK_CAT_ID, RISK_CAT_NM, 
                           RISK_CAT_RK_, SOURCE_SYSTEM_CD);

   %put STATUS;
   %put STATUS Handling processes dataset;
   %orvl_migrate_hier_item(&internal_dir..RM_PROCESS_DIM, &Output_lib..process_structure, 
                           Process_ID, Process_Name, 
                           PROCESS_RK, PROCESS_ID, PROCESS_NM, 
                           PROCESS_RK_, SOURCE_SYSTEM_CD);


   %put STATUS;
   %put STATUS Handling causes dataset;
   %orvl_migrate_hier_item_flatten(&internal_dir..RM_CAUSE_DIM, &Output_lib..cause, 
                                   Cause_ID, Cause_Name, 
                                   CAUSE_RK, CAUSE_ID, CAUSE_NM, 
                                   SOURCE_SYSTEM_CD);

   %put STATUS;
   %put STATUS Handling countries dataset;
   %orvl_migrate_hier_item_flatten(&internal_dir..RM_GEOGRAPHY_DIM, &Output_lib..country, 
                                   Country_ID, Country_Name, 
                                   GEOGRAPHY_RK, GEOGRAPHY_ID, GEOGRAPHY_NM, 
                                   SOURCE_SYSTEM_CD);

   /*
   %let InputDs = &internal_dir..RM_KRI_DIM;
   %put STATUS;
   %put STATUS Handling risk factor variables dataset;
   data &Output_lib..risk_factor_variable (keep=Risk_Factor_Variable_ID
                                                Risk_Factor_Variable_Name
                                                Upper_Bound
                                                Lower_Bound);
      set &internal_dir..RM_KRI_DIM (rename=(KRI_RK=in_rk
                                             KRI_ID=in_id
                                             KRI_NAME=in_name
                                             KRI_SOURCE_SYSTEM_CD=in_src))
         end=LAST;
      length RISK_FACTOR_VARIABLE_NAME $100. msg $200.;

      if in_rk eq . then do;
         msg = "STATUS WARNING: Obs " || TRIM(LEFT(put(_N_, best12.))) || " of &InputDs. has a missing Id.  Dropping the observation.";
         put msg;
      end;
      else do;
         Risk_Factor_Variable_ID = in_rk;
         Risk_Factor_Variable_Name = TRIM(LEFT(in_src)) || '_' || TRIM(LEFT(in_id));
         if TRIM(LEFT(in_name)) ne '' then do;
            Risk_Factor_Variable_Name = TRIM(LEFT(in_name)) || '_' || Risk_Factor_Variable_Name;
         end;
         output &Output_lib..risk_factor_variable;
      end;
   run;
   */


   %let InputDs = &internal_dir..RM_INSURANCE_POLICY_DIM;
   %put STATUS;
   %put STATUS Handling insurance dataset;
   data &Output_lib..insurance (keep=Insurance_ID
                                     Insurance_Name
                                     Premium
                                     Deductible
                                     Limit
                                     Aggregate_Limit
                                     Insurer_Rating
                                     Effective_From_Date
                                     Effective_To_Date
                                     Cancel_Notice_Period_In_Days
                                     Payout_Haircut);
      set &internal_dir..RM_INSURANCE_POLICY_DIM (rename=(INSURANCE_POLICY_RK=in_rk
                                                          POLICY_NO=in_policy_num
                                                          POLICY_SHORT_DESC=in_desc_short
                                                          POLICY_DESC=in_desc))
         end=LAST;
      length Insurance_Name $100. msg $200.;

      if in_rk eq . then do;
         msg = "STATUS WARNING: Obs " || TRIM(LEFT(put(_N_, best12.))) || " of &InputDs. has a missing Id.  Dropping the observation.";
         put msg;
      end;
      else do;
         Insurance_ID = in_rk;
         Insurance_Name = TRIM(LEFT(in_policy_num)) || " - " || TRIM(LEFT(in_desc_short));
         Premium = 0;
         Deductible = 0;
         limit = 1;
         Aggregate_limit=2;
         insurer_rating = .;
         Effective_From_date = '06Mar62'd;
         Effective_To_Date = '06Mar09'd;
         Cancel_Notice_period_in_Days = 30;
         payout_haircut = .6;
         output &Output_lib..insurance;
      end;
   run;


   %let InputDs = &internal_dir..RM_OP_RISK_EVENT;
   %put STATUS;
   %put STATUS Handling shared loss group dataset;
   data &Output_lib..shared_loss_group (keep=Shared_Group_ID
                                             Shared_Group_Code
                                             Shared_Group_Name);
      set &internal_dir..RM_OP_RISK_EVENT (rename=(OP_RISK_EVENT_RK=in_rk
                                    OP_RISK_EVENT_ID=in_id
                                    EVENT_SHORT_DESC=in_desc_short))
         end=LAST;
      length Shared_Group_Name $100. Shared_Group_Code $15. msg $200.;

      if in_rk eq . then do;
         msg = "STATUS WARNING: Obs " || TRIM(LEFT(put(_N_, best12.))) || " of &InputDs. has a missing Id.  Dropping the observation.";
         put msg;
      end;
      else do;
         Shared_Group_ID = in_rk;
         Shared_Group_Code = in_id;
         Shared_Group_Name = in_desc_short;
         output &Output_lib..shared_loss_group;
      end;
   run;


   %let InputDs = &internal_dir..RM_FINANCIAL_IMPACT;
   %put STATUS;
   %put STATUS Handling loss event dataset;
   %put STATUS Preprocessing estimates;
   data work._RM_FINANCIAL_IMPACT; 
      set &internal_dir..RM_FINANCIAL_IMPACT; 
      if (IMPACT_TYPE_CD eq 'LOSS') then do;
         if (((IMPACT_LOSS_AMT = 0) or (IMPACT_LOSS_AMT = .)) and (IMPACT_FINANCIAL_STATUS_CD eq 'EST')) then do;
            IMPACT_LOSS_AMT = IMPACT_EXPECTED_AMT;
         end;
      end;
   run;
   %put STATUS Sorting losses;
   proc sort data=work._RM_FINANCIAL_IMPACT out=work._RM_FINANCIAL_IMPACT_SORT;
      by OP_RISK_EVENT_RK;
   run;
   %put STATUS Summing losses;
   proc summary data=work._RM_FINANCIAL_IMPACT_SORT;
      by OP_RISK_EVENT_RK;
         output sum=SUM_IMPACT_LOSS_AMT SUM_IMPACT_RECOVERY_AMT SUM_IMPACT_INSURANCE_AMT SUM_IMPACT_NON_INSURANCE_AMT
             out= work._RM_FINANCIAL_IMPACT_SUM;
      var IMPACT_LOSS_AMT IMPACT_RECOVERY_AMT IMPACT_INSURANCE_AMT IMPACT_NON_INSURANCE_AMT;
   run;
   %put STATUS Merging summary and losses;
   data work._RM_FINANCIAL_IMPACT_MERGED; 
      merge work._RM_FINANCIAL_IMPACT_SORT work._RM_FINANCIAL_IMPACT_SUM; 
      by OP_RISK_EVENT_RK;
      if (IMPACT_TYPE_CD eq 'LOSS') then do;
         weighted_amt = IMPACT_LOSS_AMT / SUM_IMPACT_LOSS_AMT;
         if (((weighted_amt = 0) or (weighted_amt = .)) and (IMPACT_FINANCIAL_STATUS_CD eq 'EST')) then do;
            weighted_amt = IMPACT_EXPECTED_AMT;
         end;
         IMPACT_INSURANCE_AMT = weighted_amt * SUM_IMPACT_INSURANCE_AMT;
         IMPACT_NON_INSURANCE_AMT = weighted_amt * SUM_IMPACT_NON_INSURANCE_AMT;
      end;
   run;
   %put STATUS Extracting losses;
   data &Output_lib..loss_event_detail (keep=Loss_Event_ID
                                             Reference_ID
                                             GL_Code
                                             Description_of_Event
                                             Internal_BL_ID
                                             Internal_RC_ID
                                             Process_ID
                                             Country_of_Legal_Entity_ID
                                             Country_of_Incident_ID
                                             First_Date_of_Event
                                             Last_Date_of_Event
                                             Date_of_Discovery
                                             Date_of_Provision
                                             Date_of_Settlement
                                             Insurance_Recovery_Date_1
                                             Insurance_Recovery_Date_2
                                             Non_Insurance_Recovery_Date
                                             Shared_Loss
                                             Shared_Loss_Group_ID
                                             Source_of_Information
                                             Gross_Loss_Amount
                                             Non_Insurance_Recoveries
                                             Insurance_Recovery_1
                                             Insurance_Recovery_2
                                             Legal_Liability
                                             Regulatory_Action
                                             Restitution
                                             Loss_or_Damage_to_Assets
                                             Loss_of_Recourse
                                             Write_Downs
                                             Financial_Status
                                             Entered_on_Date
                                             Entered_by_User_Name
                                             Threshold
                                             Insurance_1
                                             Insurance_2);
      set work._RM_FINANCIAL_IMPACT_MERGED (rename=(BL_RK=in_bl_rk
                                                    CREDIT_LOSS_FLG=in_credit_loss_flag
              /* !!!!Keep in sync with variable checker above!!!! */
                                                    DISCOVERY_DT=in_impact_discovery_dt
                                                    EVENT_DISCOVERY_DT=in_event_discovery_dt
                                                    EVENT_END_DT=in_event_end_dt
                                                    EVENT_START_DT=in_event_start_dt
                                                    EVENT_THRESHOLD=in_event_threshold
                                                    FINANCIAL_IMPACT_ID=in_financial_impact_id
              /* !!!!Keep in sync with variable checker above!!!! */
                                                    FINANCIAL_IMPACT_RK=in_sk
                                        GEOGRAPHY_RK=in_geography_rk
                                                    IMPACT_DESC=in_desc
                                                    IMPACT_END_DT=in_impact_end_dt
                                                    IMPACT_EXPECTED_AMT=in_impact_expected_amt
                                                    IMPACT_FINANCIAL_STATUS_CD=in_financial_status_cd
              /* !!!!Keep in sync with variable checker above!!!! */
                                                    IMPACT_INSURANCE_AMT=in_ins_claim_amt
                                                    IMPACT_LOSS_AMT=in_detail_loss_amt
                                                    IMPACT_NON_INSURANCE_AMT=in_non_ins_amt
                                                    IMPACT_PROVISIONED_AMT=in_provision_amt
                                                    IMPACT_RECOVERY_AMT=in_detail_recovery_amt
                                                    IMPACT_START_DT=in_impact_start_dt
              /* !!!!Keep in sync with variable checker above!!!! */
                                                    IMPACT_TYPE_CD=in_impact_type_cd
                                                    LAST_UPDATE_USER_RK=in_last_update_user_rk
                                        LEGAL_ORG_RK=in_legal_org_rk
                                        MANAGEMENT_ORG_RK=in_management_org_rk
                                                    OP_RISK_EVENT_RK=in_op_risk_event_rk
              /* !!!!Keep in sync with variable checker above!!!! */
                                                    ORIGINATOR_USER_RK=in_originator_user_rk
                                        PROCESS_RK=in_process_rk
                                        RISK_CAT_RK=in_risk_cat_rk
                                                    SOURCE_SYSTEM_CD=in_source_system_cd
                                                    THRESHOLD=in_threshold
              /* !!!!Keep in sync with variable checker above!!!! */
                                                    VALIDATION_STATE=in_validation_state))
         end=LAST;
      length WHO $32. REASON $200.;
      length ORVAR_REASON_OBS_PREFIX $40.;
      length tempRes $200.;
      format Loss_Event_ID best12.;
      format Reference_ID $15.;
      format GL_Code $20.;
      format Description_of_Event $1024.; 
      format Internal_BL_ID best12.;
      format Internal_RC_ID best12.;
      format Process_ID  best12.;
      format Country_of_Legal_Entity_ID best12.;
      format Country_of_Incident_ID best12.;
      format First_Date_of_Event mmddyy10.;
      format Last_Date_of_Event mmddyy10.;
      format Date_of_Discovery mmddyy10.;
      format Date_of_Provision mmddyy10.; 
      format Date_of_Settlement mmddyy10.;
      format Insurance_Recovery_Date_1  mmddyy10.;
      format Insurance_Recovery_Date_2  mmddyy10.;
      format Non_Insurance_Recovery_Date mmddyy10.;
      format Shared_Loss $1.;
      format Shared_Loss_Group_ID best12.;
      format Source_of_Information $100.;
      format Gross_Loss_Amount best12.;
      format Non_Insurance_Recoveries best12.;
      format Insurance_Recovery_1 best12.;
      format Insurance_Recovery_2 best12.;
      format Legal_Liability best12.;
      format Regulatory_Action best12.;
      format Restitution best12.;
      format Loss_or_Damage_to_Assets best12.;
      format Loss_of_Recourse best12.;
      format Write_Downs best12.;
      format Financial_Status $1.;
      format Entered_on_Date mmddyy10.;
      format Entered_by_User_Name $100.;
      format Threshold best12.;
      format Insurance_1 best12.;
      format Insurance_2 best12.;
      retain num_dropped;

      if _N_ = 1 then do;
         maxIndex = 1;
         declare hash inHash(dataset: "&internal_dir..RM_OP_RISK_EVENT", hashexp: 10);
         rc = inHash.defineKey("OP_RISK_EVENT_RK");
         rc = inHash.defineData("EVENT_LOSS_CNT");
         rc = inHash.defineDone();
         call missing(OP_RISK_EVENT_RK, EVENT_LOSS_CNT);
         num_dropped = 0;
      end;

      Loss_Event_ID = in_sk;
      Reference_ID = in_financial_impact_id;
      /**GL_Code = in_ledger_cd;*/
      Description_of_Event = in_desc;
      Internal_BL_ID = in_bl_rk;
      Internal_RC_ID = in_risk_cat_rk;
      Process_ID = in_process_rk;
      Country_of_Legal_Entity_ID = in_geography_rk;
      Country_of_Incident_ID = in_geography_rk;
      First_Date_of_Event = in_impact_start_dt;
      if First_Date_of_Event eq . then First_Date_of_Event = in_event_start_dt;
      Last_Date_of_Event = in_impact_end_dt;
      if Last_Date_of_Event eq . then Last_Date_of_Event = in_event_end_dt;
      if Last_Date_of_Event eq . then Last_Date_of_Event = First_Date_of_Event;
      Date_of_Discovery = in_impact_discovery_dt;
      if Date_of_Discovery eq . then Date_of_Discovery = in_event_discovery_dt;
      /**Date_of_Provision = in_provision_dt;*/
      /**Date_of_Settlement = in_settlement_dt;*/
      /**Insurance_Recovery_Date_1 = in_ins_claim_dt;*/
      Insurance_Recovery_Date_2 = .;
      Non_Insurance_Recovery_Date = .;
      Shared_Loss = 'N';
      Shared_Loss_Group_ID = in_op_risk_event_rk;
      Source_of_Information = in_source_system_cd;
      Gross_Loss_Amount = in_detail_loss_amt; 
      Non_Insurance_Recoveries = in_non_ins_amt;
      Insurance_Recovery_1 = in_ins_claim_amt;
      Insurance_Recovery_2 = .;
      Legal_Liability = .;
      Regulatory_Action = .;
      Restitution = .;
      Loss_or_Damage_to_Assets = .;
      Loss_of_Recourse = .;
      Write_Downs = .;
      /*Financial_Status =var29;*/
      /**Entered_on_Date = in_entered_on_dt;*/
      Entered_by_User_Name = in_originator_user_rk;
      Threshold = in_threshold;
      if Threshold eq . then Threshold = in_event_threshold;
      if Threshold eq . then Threshold = 0;
      /**Insurance_1 = in_insurance_policy_rk;*/
      Insurance_2 = .;

      OP_RISK_EVENT_RK = in_op_risk_event_rk;
      rc = inHash.find();
      if (rc ^= 0) or (EVENT_LOSS_CNT < 2) then do;
         Shared_Loss_Group_ID = .; /* Referenced item not found.  Don't output the data. */
      end;
      else do;
         Shared_Loss = 'Y'; /* There is more than 1 loss for the event. */
      end;
      
      if upcase(in_impact_type_cd) eq 'GAIN' then do;
         in_financial_status_cd = 'NRM';
         tempRes = "STATUS WARNING: Obs " || put(_N_, Z7.) || " - This is a gain, so it will be treated as a Near Miss.";
         put tempRes;
         tempRes = "STATUS ! Changing Financial Status to Near Miss, and making the observation a loss.";
         put tempRes;
      end;

      if in_financial_status_cd eq '' then do;
         in_financial_status_cd = 'PRO';
      end;

      if in_financial_status_cd ne '' then do; 
         in_financial_status_cd = upcase(in_financial_status_cd); 
         if in_financial_status_cd eq 'SET' then do;
            Financial_Status = 'F';
            if Date_of_Settlement eq . then do;
               Date_of_Settlement = Last_Date_of_Event;
               tempRes = "STATUS WARNING: Obs " || put(_N_, Z7.) || " - Date of Settlement is missing.  Setting it to Last Date of Event.";
               put tempRes;
            end;
         end;
         if in_financial_status_cd eq 'PRO' or in_financial_status_cd eq '' then do;
            if in_financial_status_cd eq '' then do;
               tempRes = "STATUS WARNING: Obs " || put(_N_, Z7.) || " - Financial Status missing. Setting it to Provisioned.";
               put tempRes;
            end;
            Financial_Status = 'P';
            if Date_of_Provision eq . then do;
               if (Date_of_Settlement ne .) then do;
                  Date_of_Provision = Last_Date_of_Event;
                  tempRes = "STATUS WARNING: Obs " || put(_N_, Z7.) || " - An event with financial status of 'P' cannot have a Date of Settlement,";
                  put tempRes;
                  tempRes = "STATUS ! and Date of Provision is missing.  Setting Date of Provision to Date of Settlement.";
                  put tempRes;
               end;
               else do;
                  Date_of_Provision = Last_Date_of_Event;
                  tempRes = "STATUS WARNING: Obs " || put(_N_, Z7.) || " - Date of Provision is missing.  Setting it to Last Date of Event.";
                  put tempRes;
               end;
            end;
         end;
         else if in_financial_status_cd eq 'NRM' or in_financial_status_cd eq 'EST' or in_financial_status_cd eq 'CST' then do;
            if in_financial_status_cd eq 'NRM' then do;
               Financial_Status = 'N';
            end;
            else do;
               Financial_Status = 'E';
            end;
            if ((left(trim(Financial_Status)) = 'E') and ((Gross_Loss_Amount = .) or (Gross_Loss_Amount = 0))) then do;
               Gross_Loss_Amount = in_impact_expected_amt;
            end;
            if Date_of_Settlement ne . and Date_of_Discovery eq . then do;
               Date_of_Discovery = Date_of_Settlement;
               Date_of_Settlement = .;
               tempRes = "STATUS WARNING: Obs " || put(_N_, Z7.) || " - An event with financial status of 'N' or 'E' cannot have a Date of Settlement,";
               put tempRes;
               tempRes = "STATUS ! and Date of Discovery is missing.  Setting Date of Discovery to Date of Settlement, and setting Date of Settlement to missing.";
               put tempRes;
            end;
            if Date_of_Provision ne . then do;
               if  Date_of_Discovery eq . then do;
                  Date_of_Discovery = Date_of_Provision;
                  tempRes = "STATUS WARNING: Obs " || put(_N_, Z7.) || " - An event with financial status of 'N' or 'E' cannot have a Date of Provision,";
                  put tempRes;
                  tempRes = "STATUS ! and Date of Discovery is missing.  Setting Date of Discovery to Date of Provision, and setting Date of Provision to missing.";
                  put tempRes;
               end;
               else do;
                  tempRes = "STATUS WARNING: Obs " || put(_N_, Z7.) || " - An event with financial status of 'N' or 'E' cannot have a Date of Provision.";
                  put tempRes;
                  tempRes = "STATUS ! Setting Date of Provision to missing.";
                  put tempRes;
               end;
               Date_of_Provision = .;
            end;
            else if Date_of_Discovery eq . then do;
               Date_of_Discovery = Last_Date_of_Event;
               tempRes = "STATUS WARNING: Obs " || put(_N_, Z7.) || " - Date of Discovery is missing.  Setting it to Last Date of Event.";
               put tempRes;
            end;
         end;
      end;

      num_dropped = num_dropped + 1;

      if upcase(in_credit_loss_flag) eq 'Y' then do;
         tempRes = "STATUS WARNING: Obs " || put(_N_, Z7.) || " - Credit loss.  Dropping the observation.";
         put tempRes;
      end;
      else if upcase(in_validation_state) ne 'VALID' then do;
         tempRes = "STATUS WARNING: Obs " || put(_N_, Z7.) || " - Not marked VALID.  Dropping the observation.";
         put tempRes;
      end;
      else if Financial_Status eq '' then do;
         tempRes = "STATUS WARNING: Obs " || put(_N_, Z7.) || " - Missing financial status.  Dropping the observation.";
         put tempRes;
      end;
      else if Last_Date_of_Event eq . then do;
         tempRes = "STATUS WARNING: Obs " || put(_N_, Z7.) || " - Missing dates of the event.  Dropping the observation.";
         put tempRes;
      end;
      else if Internal_BL_ID eq . then do;
         tempRes = "STATUS WARNING: Obs " || put(_N_, Z7.) || " - Missing business line reference.  Dropping the observation.";
         put tempRes;
      end;
      else if Internal_RC_ID eq . then do;
         tempRes = "STATUS WARNING: Obs " || put(_N_, Z7.) || " - Missing risk category reference.  Dropping the observation.";
         put tempRes;
      end;
      else if upcase(in_impact_type_cd) eq 'RECOV' then do;
         tempRes = "STATUS WARNING: Obs " || put(_N_, Z7.) || " - Recovery.  Merged into the matching losses.  Dropping the observation.";
         put tempRes;
      end;
      else do;
         output;
         num_dropped = num_dropped - 1;
      end;

      if (LAST = 1) then do;
         if (num_dropped > 0) then do;
            put "STATUS";
            put "STATUS ========";
            tempRes = "STATUS WARNING: Dropped " || put(num_dropped, Z7.) || " of " || put(_N_, Z7.) || " losses read.  See above for details.";
            put tempRes;
            put "STATUS ========";
            put "STATUS";
         end;
      end;
   run;

   proc datasets library=work memtype=data nolist;
      delete _RM:;
   run; 
   quit;

   %orvl_ds_exists(&Output_lib..loss_event_detail, "&TYP_NM", false);
   %if &ds_nobs eq 0 %then %do;
      data _null_;
         call symput("LOAD_STATUS", 3);
         call symput("LOAD_REASON", "No losses were able to be successfully transformed.  See the log for details.");
      run;
   %end;

%DONE:
   %orvl_cmn_status_processing();

   %if %TRIM(%LEFT(&LOAD_STATUS)) eq 0 %then %do;
      data _null_;
         call symput("LOAD_REASON", "Completed conversion of Monitor datasets to input datasets.");
      run;
      proc sql;
         UPDATE LOAD_STATUS_DETAILS SET STATUS = &LOAD_STATUS, REASON = "&LOAD_REASON" WHERE WHO = "MAIN";
         UPDATE &outstatus SET status = 0, reason = "";
      quit;
   %end;
   %else %do;
      proc sql;
         UPDATE LOAD_STATUS_DETAILS SET STATUS = &LOAD_STATUS, REASON = "&LOAD_REASON" WHERE WHO = "MAIN";
         UPDATE &outstatus SET status = &LOAD_STATUS, reason = "&LOAD_REASON";
      quit;
   %end;
   
   %put STATUS;

   %put STATUS &LOAD_REASON;

   %put STATUS;

   %put STATUS PROCESS END;

   %orvl_cmn_global_cleanup();

   /*- restore syntaxcheck options -*/
   options &syntaxopt;

%mend ORVL_MIGRATE_MONITOR_DS;
